
-- AUTOR      : Miguel A. Serna Mata
-- ASIGNATURA : Administracin de bases de datos
-- CURSO      : 3 Informtica de Gestin
-- EMAIL      : masernamata@universia.es
-- FECHA      : 24-Dic-2002
-- CATEGORA  : Programacin PL/SQL
-- CONTENIDO
--   Un script con un ejemplo de utilizacin de la funcin ffCreateTable.
-- OBSERVACIONES
--   En el fichero INIT.ORA, escribir lo siguiente:
--     UTL_FILE_DIR=*
--   y despus inicializar (startup) de nuevo la base de datos.
--   Ejecutar con el comando START de SQL*Plus.

set serveroutput on
set echo off
set show off
set verify off

prompt  ---------------------------------------------------

prompt  Ejemplo de utilizacin de la funcin ffCreateTable.

prompt  ---------------------------------------------------

prompt  1. Creamos las tablas: Paises, Regiones, Censo, Actividades y Adoptadas.

PAUSE Pulse RETURN para continuar...

CREATE TABLE Paises (
  codpais VARCHAR2(4) CONSTRAINT pk_codpais_pais PRIMARY KEY,
  nompais VARCHAR2(15) CONSTRAINT u_nompais_pais UNIQUE,
  nomcapital VARCHAR2(15),
  presup_social NUMBER(6) CONSTRAINT presup_social_positivo
				CHECK(presup_social>=0)
);

CREATE TABLE Regiones (
  codpais CONSTRAINT fk_codpais_regiones REFERENCES Paises(codpais),
  nomregion VARCHAR2(15),
  extension NUMBER(6) CONSTRAINT extension_regiones CHECK(extension>0),
  cgini NUMBER(5,2) CONSTRAINT cgini_positivo CHECK(cgini>=0),
  poblacion NUMBER(6) CONSTRAINT poblacion_regiones CHECK(poblacion>0),
  umbral_pobreza NUMBER(9,1) CONSTRAINT umbral_probreza_regiones CHECK(umbral_pobreza>=0),
  CONSTRAINT pk_codpais_nomregion PRIMARY KEY(codpais,nomregion)
);

CREATE TABLE Censo (
  codpersona NUMBER(6),
  codpaisnac CONSTRAINT fk_codpaisnac_censo REFERENCES Paises(codpais),
  codpaisvive,
  nomregion,
  nompersona VARCHAR2(15),
  sexo VARCHAR2(1) CONSTRAINT sexo_V_o_H_censo CHECK(sexo='V' OR sexo='H'),
  CONSTRAINT fk_codpaisvive_y_nomregion FOREIGN KEY(codpaisvive,nomregion)
				REFERENCES Regiones(codpais,nomregion),
  ingresos NUMBER(6) CONSTRAINT ingresos_positivos_censo CHECK(ingresos>=0),
  CONSTRAINT pk_codpaisnac_y_codpersona PRIMARY KEY(codpaisnac,codpersona)
);

CREATE TABLE Actividades (
  codactividad NUMBER(3) CONSTRAINT pk_codactividad_actividades PRIMARY KEY,
  nomactividad VARCHAR2(15) CONSTRAINT u_nomactividad_actividades UNIQUE,
  costo NUMBER(6) CONSTRAINT costo_positivo CHECK(costo>=0)
);

CREATE TABLE Adoptadas (
  codpais,
  nomregion,
  codactividad CONSTRAINT fk_codactividad_adoptadas REFERENCES Actividades(codactividad),
  fechaini DATE,
  duracion NUMBER(3) CONSTRAINT duracion_positiva CHECK (duracion>0),
  CONSTRAINT fk_codpais_y_nomregion FOREIGN KEY (codpais,nomregion)
				REFERENCES Regiones(codpais,nomregion),
  CONSTRAINT pk_codpais_y_nomreg_y_codact
				PRIMARY KEY(codpais,nomregion,codactividad)
);

prompt  2. Creamos la tabla Tablas, donde insertamos los nombres de

prompt  las cinco tablas creadas.

PAUSE Pulse RETURN para continuar...

CREATE TABLE Tablas (
 nombre  VARCHAR2(20)
);

INSERT INTO Tablas
VALUES ('Paises');
INSERT INTO Tablas
VALUES ('Regiones');
INSERT INTO Tablas
VALUES ('Censo');
INSERT INTO Tablas
VALUES ('Actividades');
INSERT INTO Tablas
VALUES ('Adoptadas');
COMMIT;

prompt  3. Suponemos que queremos crear esas mismas tablas

prompt  para otro usuario, pero no tenemos las sentencias CREATE TABLE.

prompt  En este punto nos es til la funcin ffCreateTable.

prompt  Vamos a guardar las rdenes CREATE TABLE de las cinco tablas

prompt  en un fichero.

ACCEPT directorio CHAR PROMPT 'Nombre del Directorio: '
ACCEPT fichero CHAR PROMPT 'Nombre del Fichero: '

DECLARE
  f  utl_file.file_type;

  CURSOR busca_tabla IS
    SELECT * FROM tablas;

  var_nombre_tabla  tablas.nombre%type;
BEGIN
  -- Abrimos el fichero con la opcin de escritura ('w')
  f:= utl_file.fopen('&directorio','&fichero','w');

  -- Guardamos la sentencia de creacin de cada tabla
  OPEN busca_tabla;
  LOOP
    FETCH busca_tabla INTO var_nombre_tabla;
    EXIT WHEN busca_tabla%notfound;
    -- Insertamos un comentario con el nombre de la tabla
    utl_file.putf(f,'\n-- Tabla: %s\n',var_nombre_tabla);
    ffCreateTable(var_nombre_tabla,f);
  END LOOP;
  CLOSE busca_tabla;

  -- Cerramos el fichero
  utl_file.fclose(f);

EXCEPTION
  WHEN utl_file.invalid_path THEN
    dbms_output.put_line(' Error: Directorio no vlido.');
  WHEN others THEN
    dbms_output.put_line(' Error: ' || SQLERRM);
END;
/

prompt 4. Las tablas son borradas.

PAUSE Pulse RETURN para continuar...

DROP TABLE Censo;
DROP TABLE Adoptadas;
DROP TABLE Actividades;
DROP TABLE Regiones;
DROP TABLE Paises;
DROP TABLE Tablas;

prompt 5. Si todo ha ido bien, comprueba que el fichero &fichero, del directorio

prompt &directorio, contiene las rdenes CREATE TABLE correctas.

PAUSE Pulse RETURN para terminar...