
-- AUTOR      : Miguel A. Serna Mata
-- ASIGNATURA : Administracin de bases de datos
-- CURSO      : 3 Informtica de Gestin
-- EMAIL      : masernamata@universia.es
-- FECHA      : 24-Dic-2002
-- CATEGORA  : Programacin PL/SQL
-- CONTENIDO
--     Dos procedimientos, ffCreateTable y fCreateTable, que permiten construir
--   la orden CREATE TABLE necesaria para crear una copia de la tabla pasada 
--   como parmetro.
-- OBSERVACIONES
--     La seguridad del servidor para un fichero PL/SQL de entrada/salida consiste en
--   restringir los directorios que pueden ser accedidos. Los directorios accesibles
--   deben ser especificados en el fichero INIT.ORA. En la versin 8.1.5 de Oracle
--   para Windows 98, este fichero se encuentra en c:\Oracle\ADMIN\orcl\PFILE (suponemos 
--   que Oracle est instalado en la unidad c:).
--     Los directorios accesibles para la funciones del paquete UTL_FILE, utilizadas
--   por las funciones implementadas (ffCreateTable y fCreateTable), se especifican
--   en el fichero de inicializacin usando el parmetro UTL_FILE_DIR:
--   UTL_FILE_DIR = <nombre_directorio>.
--     Si se quiere hacer accesible cualquier directorio, hay que escribir UTL_FILE_DIR=*
--     Para ms informacin, consultar el manual de Oracle.


/**********************************************************/
/* ffCreateTable ******************************************/
/**********************************************************/
/* Guarda en el fichero del manejador f, la               */
/* sentencia CREATE TABLE correspondiente a la tabla      */
/* var_nombre_tabla.                                      */
/**********************************************************/
/* Excepciones:                                           */
/*  -20000 -- La tabla no existe                          */
/*  -20002 -- Error (cualquier otro)                      */
/**********************************************************/
CREATE OR REPLACE PROCEDURE ffCreateTable
  (var_nombre_tabla       IN  user_tables.table_name%type,
   f                      IN OUT utl_file.file_type) IS

  /*** Para los atributos ***/
  CURSOR busca_columnas IS
    SELECT *
    FROM user_tab_columns
    WHERE UPPER(table_name)=UPPER(var_nombre_tabla);

  var_col     user_tab_columns%rowtype;

  /*** Para las restricciones ***/
  var_restriccion    user_constraints.constraint_name%type;
  var_campo          user_cons_columns.column_name%type;
  var_condicion      user_constraints.search_condition%type;
  var_references     user_constraints.r_constraint_name%type;
  var_tabla_ref      user_constraints.constraint_name%type;

  CURSOR busca_primaria IS
     SELECT constraint_name
     FROM user_constraints
     WHERE UPPER(table_name)=UPPER(var_nombre_tabla)
       AND constraint_type='P';

  CURSOR busca_unique IS
     SELECT constraint_name
     FROM user_constraints
     WHERE UPPER(table_name)=UPPER(var_nombre_tabla)
       AND constraint_type='U';

  CURSOR busca_check IS
     SELECT search_condition
     FROM user_constraints
     WHERE UPPER(table_name)=UPPER(var_nombre_tabla)
       AND constraint_type='C';

  CURSOR busca_foranea IS
     SELECT constraint_name, r_constraint_name
     FROM user_constraints
     WHERE UPPER(table_name)=UPPER(var_nombre_tabla)
       AND constraint_type='R';

  CURSOR busca_campos IS
     SELECT column_name
     FROM user_cons_columns
     WHERE constraint_name=var_restriccion;

  CURSOR busca_campos_ord
     (p_restriccion   user_constraints.constraint_name%type) IS
     SELECT column_name
     FROM user_cons_columns
     WHERE constraint_name=p_restriccion
     ORDER BY position;


  /*** Para comprobar si la tabla existe */
  var_existe    NUMBER(1);


BEGIN  
  -- Si la tabla no existe, generamos una excepcin 
  SELECT COUNT(*) INTO var_existe
  FROM user_tables
  WHERE UPPER(table_name)=UPPER(var_nombre_tabla);
  IF var_existe=0 THEN
    RAISE_APPLICATION_ERROR(-20000,'La tabla no existe');
  END IF;

  -- Guardamos la cabecera de la sentencia 
  utl_file.putf(f,'CREATE TABLE %s (\n',var_nombre_tabla);

  /***************************  Columnas **********************************/
  OPEN busca_columnas;
  LOOP
    FETCH busca_columnas INTO var_col;
    EXIT WHEN busca_columnas%NOTFOUND;

    -- Guarda el nombre del atributo 
    IF busca_columnas%rowcount=1 THEN
      utl_file.putf(f,' %s',LOWER(var_col.column_name));
    ELSE
      utl_file.putf(f,',%s',LOWER(var_col.column_name));
    END IF;

    -- Guardamos el tipo
    IF UPPER(var_col.data_type)='NUMBER' THEN
      utl_file.putf(f,' %s(%s,%s)',var_col.data_type,
                                   var_col.data_precision,
                                   var_col.data_scale);
    ELSIF UPPER(var_col.data_type) IN ('VARCHAR2','NVARCHAR2','RAW','UROWID',
                                       'CHAR','NCHAR') THEN
      utl_file.putf(f,' %s(%s)',var_col.data_type,
                                var_col.data_length);
    ELSE
      utl_file.putf(f,' %s',var_col.data_type);
    END IF;

    -- Guardamos el valor por defecto, si tiene
    IF var_col.data_default IS NOT NULL THEN
      utl_file.putf(f,' DEFAULT %s',var_col.data_default);
    END IF;

    -- Salto de lnea 
    utl_file.putf(f,'\n');
  END LOOP;
  CLOSE busca_columnas;


  /**** Restricciones: *******************************/

  /**** PKs *****************************************/
  OPEN busca_primaria;
  LOOP
    FETCH busca_primaria INTO var_restriccion;
    EXIT WHEN busca_primaria%notfound;

    OPEN busca_campos;
    LOOP
      FETCH busca_campos INTO var_campo;
      EXIT WHEN busca_campos%notfound;
      IF busca_campos%rowcount=1 THEN
        utl_file.putf(f,',PRIMARY KEY (%s',LOWER(var_campo));
      ELSE
        utl_file.putf(f,',%s',LOWER(var_campo));
      END IF;
    END LOOP;
    CLOSE busca_campos;
    
    -- Parntesis y Salto de lnea
    utl_file.putf(f,')\n');
  END LOOP;
  CLOSE busca_primaria;

  /**** UNIQUEs ************************************/
  OPEN busca_unique;
  LOOP
    FETCH busca_unique INTO var_restriccion;
    EXIT WHEN busca_unique%notfound;

    OPEN busca_campos;
    LOOP
      FETCH busca_campos INTO var_campo;
      EXIT WHEN busca_campos%notfound;
      IF busca_campos%rowcount=1 THEN
        utl_file.putf(f,',UNIQUE (%s',LOWER(var_campo));
      ELSE
        utl_file.putf(f,',%s',LOWER(var_campo));
      END IF;
    END LOOP;
    CLOSE busca_campos;
    
    -- Parntesis y Salto de lnea
    utl_file.putf(f,')\n');
  END LOOP;
  CLOSE busca_unique;


  /**** CHECKs ************************************/
  OPEN busca_check;
  LOOP
    FETCH busca_check INTO var_condicion;
    EXIT WHEN busca_check%notfound;

    utl_file.putf(f,',CHECK (%s) \n',var_condicion); 
  END LOOP;
  CLOSE busca_check;

  /**** FKs ***************************************/
  OPEN busca_foranea;
  LOOP
    FETCH busca_foranea INTO var_restriccion, var_references;
    EXIT WHEN busca_foranea%notfound;
   
    -- Escribe los atributos de la fk
    OPEN busca_campos_ord(var_restriccion);
    LOOP
      FETCH busca_campos_ord INTO var_campo;
      EXIT WHEN busca_campos_ord%notfound;

      IF busca_campos_ord%rowcount=1 THEN
        utl_file.putf(f,',FOREIGN KEY (%s',LOWER(var_campo));
      ELSE
        utl_file.putf(f,',%s',LOWER(var_campo));
      END IF;
    END LOOP;
    CLOSE busca_campos_ord;
    
    -- Cierra parntesis
    utl_file.putf(f,')');

    -- Obtiene la tabla base de la fornea 
    SELECT table_name INTO var_tabla_ref
    FROM user_constraints
    WHERE constraint_name=var_references;

    -- Guardamos la tabla a la que referencia
    utl_file.putf(f,' REFERENCES %s',LOWER(var_tabla_ref));

    -- Escribimos los atributos referenciados
    OPEN busca_campos_ord(var_references);
    LOOP
      FETCH busca_campos_ord INTO var_campo;
      EXIT WHEN busca_campos_ord%notfound;

      IF busca_campos_ord%rowcount=1 THEN
        utl_file.putf(f,'(%s',LOWER(var_campo));
      ELSE
        utl_file.putf(f,',%s',LOWER(var_campo));
      END IF;
    END LOOP;
    CLOSE busca_campos_ord;

    -- Parntesis y Salto de lnea
    utl_file.putf(f,')\n');

  END LOOP;
  CLOSE busca_foranea;

  /*** Cerramos la sentencia ***/
  utl_file.putf(f,');');

  /* Escribimos los datos pendientes */
  utl_file.fflush(f);


EXCEPTION
  WHEN others THEN
    RAISE_APPLICATION_ERROR(-20002,'Error:' || SQLERRM);
END ffCreateTable;
/


/**********************************************************/
/* fCreateTable *******************************************/
/**********************************************************/
/* Guarda en un fichero con nombre var_nombre_fichero,    */
/* localizado en el directorio var_ruta, la sentencia     */
/* CREATE TABLE correspondiente a la tabla                */
/* var_nombre_tabla.                                      */
/* Si el fichero ya existe, su contenido ser sobrescrito.*/
/**********************************************************/
/* Excepciones:                                           */
/*  -20000 -- La tabla no existe                          */
/*  -20001 -- Camino invlido                             */
/*  -20002 -- Error (cualquier otro)                      */
/**********************************************************/
CREATE OR REPLACE PROCEDURE fCreateTable
  (var_nombre_tabla       IN user_tables.table_name%type,
   var_ruta               IN VARCHAR2,
   var_nombre_fichero     IN VARCHAR2) IS

  f   utl_file.file_type;
BEGIN
  -- Abrimos el fichero
  f:=utl_file.fopen(var_ruta,var_nombre_fichero,'w');
  -- Escribimos la sentencia CREATE TABLE
  ffCreateTable(var_nombre_tabla,f);
  -- Cerramos el fichero
  utl_file.fclose(f);

EXCEPTION
  WHEN utl_file.invalid_path THEN
    RAISE_APPLICATION_ERROR(-20001,'Camino invalido.');
  WHEN others THEN
    RAISE_APPLICATION_ERROR(-20002,'Error:' || SQLERRM);
END fCreateTable;
/
